package geniusweb.boa.biddingstrategy;

import static org.junit.Assert.assertTrue;
import static org.mockito.Matchers.any;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

import java.math.BigDecimal;
import java.net.URISyntaxException;
import java.util.Arrays;

import org.junit.Before;
import org.junit.Test;

import geniusweb.actions.Action;
import geniusweb.actions.Offer;
import geniusweb.boa.BoaState;
import geniusweb.inform.Settings;
import geniusweb.issuevalue.Bid;
import geniusweb.issuevalue.DiscreteValue;
import geniusweb.profile.utilityspace.LinearAdditive;
import geniusweb.progress.Progress;
import geniusweb.references.Parameters;
import tudelft.utilities.immutablelist.FixedList;
import tudelft.utilities.immutablelist.ImmutableList;
import tudelft.utilities.logging.ReportToLogger;

public class TimeDepBidStratTest {

	// private static final String PROFILE =
	// "src/test/resources/testprofile.json";
	// private ProfileRef profileref;
	private LinearAdditive profile = mock(LinearAdditive.class);
	private TimeDependentBiddingStrategy biddingstrat;
	private ExtendedUtilSpace extUtilSpace;
	private Settings settings = mock(Settings.class);
	private BoaState state = mock(BoaState.class);

	private Bid bid1 = new Bid("iss1", new DiscreteValue("val1"));
	private Bid resBid = new Bid("iss1", new DiscreteValue("val2"));
	ImmutableList<Bid> alternatives = new FixedList<Bid>(Arrays.asList(bid1));

	@Before
	public void before() throws URISyntaxException {
		when(settings.getParameters()).thenReturn(new Parameters());
		when(state.getReporter()).thenReturn(new ReportToLogger("test"));

		biddingstrat = new TimeDependentBiddingStrategy() {
			@Override
			protected ExtendedUtilSpace getBidSpace(LinearAdditive profile) {
				return extUtilSpace;
			}

		};
		// profileref = new ProfileRef(PROFILE);

		extUtilSpace = mock(ExtendedUtilSpace.class);
		when(extUtilSpace.getMin()).thenReturn(BigDecimal.ZERO);
		when(extUtilSpace.getMax()).thenReturn(BigDecimal.ONE);

	}

	@Test
	public void smoke() {
	}

	@Test(expected = IllegalArgumentException.class)
	public void getActionTestNoProfile() {
		when(state.getSettings()).thenReturn(settings);

		biddingstrat.getAction(state);
	}

	@Test
	public void getActionAtStart() {

		when(state.getProfile()).thenReturn(profile);
		when(state.getSettings()).thenReturn(settings);

		Progress progress = mock(Progress.class);
		when(progress.get(any())).thenReturn(0d);
		// progress = right at start => utilitygoal 1.0

		when(state.getProgress()).thenReturn(progress);
		when(extUtilSpace.getBids(any())).thenReturn(alternatives);

		Action action = biddingstrat.getAction(state);

		// check that the recommender chose a bid in of utility 1.0
		verify(extUtilSpace).getBids(BigDecimal.valueOf(1.0));
		assertTrue(action instanceof Offer);
	}

	@Test
	public void getActionAtHalfway() {

		when(state.getProfile()).thenReturn(profile);
		when(state.getSettings()).thenReturn(settings);

		Progress progress = mock(Progress.class);
		when(progress.get(any())).thenReturn(0.5d);
		// progress = halfway => utilitygoal= 0.5

		when(state.getProgress()).thenReturn(progress);
		when(extUtilSpace.getBids(any())).thenReturn(alternatives);

		Action action = biddingstrat.getAction(state);

		// check that the recommender chose a bid in of utility 1.0
		verify(extUtilSpace).getBids(BigDecimal.valueOf(0.5));
		assertTrue(action instanceof Offer);
	}

	@Test
	public void getActionAtEnd() {

		when(state.getProfile()).thenReturn(profile);
		when(state.getSettings()).thenReturn(settings);

		Progress progress = mock(Progress.class);
		when(progress.get(any())).thenReturn(1d);
		// progress = end => utilitygoal= 0

		when(state.getProgress()).thenReturn(progress);
		when(extUtilSpace.getBids(any())).thenReturn(alternatives);

		Action action = biddingstrat.getAction(state);

		// check that the recommender chose a bid in of utility 1.0
		verify(extUtilSpace).getBids(BigDecimal.valueOf(0.0));
		assertTrue(action instanceof Offer);
	}

	@Test
	public void getActionAtEndWithReservation() {
		// if there is reservation bid, it should be considered min
		when(profile.getReservationBid()).thenReturn(resBid);
		when(profile.getUtility(resBid)).thenReturn(new BigDecimal("0.5"));

		when(state.getProfile()).thenReturn(profile);
		when(state.getSettings()).thenReturn(settings);

		Progress progress = mock(Progress.class);
		when(progress.get(any())).thenReturn(1d);
		// progress = end => utilitygoal= resvalue=0.6

		when(state.getProgress()).thenReturn(progress);
		when(extUtilSpace.getBids(any())).thenReturn(alternatives);

		Action action = biddingstrat.getAction(state);

		// check that the recommender chose a bid in of utility 0.5
		verify(extUtilSpace).getBids(BigDecimal.valueOf(0.5));
		assertTrue(action instanceof Offer);
	}

}
